#-*-coding:UTF-8-*-
from twisted.python import log
from twisted.words.protocols.jabber.xmlstream import IQ


#from xdata import *
#test

def x2dict(x):
	'''Creates dict in format {"var": [value1, value2...]} from jabber:x:data element.

	Note: unless the field type is list-multi, there are only 1 value - use dict["var"][0].
	
	@type x: Element
	@rtype: dict
	@return: data for Stage'''
	if not x:
		log.msg("Not x")
		return
	if x["type"] != "submit":
		log.msg('Not type="submit"')
		return
	data = {}
	log.msg(x.toXml())
	for field in x.elements():
		values = []
		if field.name != "field":
			continue
		for value in field.elements():
			if value.name != "value":
				continue
			values.append(unicode(value))
		data[field["var"]] = values 
	return data

class Stage:
	"Basic stage class, used to create derived stages with defined exec_ method."
	def __init__(self, main, id, session, data = None, xmllang=None):
		"""Sets important values for stage
@type main: MainWindow
@type id: unicode
@param id: iq id attribute that will be used in result
@type session: Session
@param: session parent Session instance
@type data: dict
@param data: generaly data generated by x2dict
@type xmllang:
@param xmllang: iq xml:lang attribute value
"""
		self.main = main
		self.session = session
		self.data = data
		self.xmllang=xmllang

		self.xform = None
		self.status = None
		self.actions = {"cancel":CancelStage} # "action":StageClass
		self.execute = None
		self.id = id

		log.msg("__init__ stage %s; data %s" % (id, data))

	def exec_(self):
		"""Define this method in derived class
You should set theese values in the derived class:
Element self.xform = jabber:x:data xml element with result or form - you can use xdata module to make this easier
str self.execute = optional <command> execute attribute
str self.status = <command> status attribute, probably "executing" or "completed"
dict self.actions = {"action":Stage} - "action" is the action chosen by responder (prev, next, execute), Stage is derived a class (not instance)"""
		pass

	def send(self, value = None):
		"Sends the result of stage with values set in __init__ and exec_"
		iq = IQ(self.main.client.xmlstream, "result")
		iq["to"] = self.session.jid
		iq["id"] = self.id
		command = iq.addElement("command")
		command.attributes = {
				"xmlns":"http://jabber.org/protocol/commands",
				"sessionid":self.session.sessionid,
				"node":self.session.node,
				"status":self.status
				}
		actions = command.addElement("actions")
		if self.execute:
			actions["execute"] = self.execute
		for action in self.actions.keys(): 
			if action != "cancel" and action != "execute":
				actions.addElement(action)
		if self.xform:
			command.addChild(self.xform)
		
		self.session.addCallbackStages(self.actions)
		iq.send()
		log.msg("Sent stage %s" % self.id)
#		self.main.client.disp(iq["id"])

class CancelStage(Stage):
	"Basic derived cancel stage"
	def exec_(self):
		self.status = "cancelled"
		self.session.sessionEnded()
		log.msg("Stage cancelled")

class Session:
	"Session class that manages stages"
	def __init__(self, main, node, firststage, jid, sessionid, fid, args):
		"""@type main: Main
@param main: Main instance
@type node: unicode
@param node: name of the session node
@type firststage: class
@param firststage: Stage class to be executed as first, next stages are defined in Stage.actions
@type jid: unicode
@param jid: full jid
@type sessionid: unicode
@param sessionid: generated automaticaly
@type fid: unicode
@param fid: first iq id attribute (for result of firststage)"""
		self.main	= main
		self.node	= unicode(node)
		self.jid 	= unicode(jid)
		self.args = args

		self.sessionid	= sessionid
		self.nextstages = {}

		self.execStage(firststage, fid)
		log.msg("Starting session %s" % self.sessionid)

	def execStage(self, stageC, id, data = None, xmllang=None):
		"Runs stageC.exec_() and stageC.send()"
		stage = stageC(self.main, id, self, data, xmllang)
		d = stage.exec_()
		log.msg("Executing stage")
		if d != None:
			d.addCallback(stage.send)
		else:
			stage.send()
		log.msg("Stage executed")

	def addCallbackStages(self, stages):
		"used in Stage.send()"
		self.nextstages = stages

	def sessionEnded(self):
		del self.main.client.commands.sessions[self.sessionid]

class Commands:
	"Class that manages commands sessions"
	def __init__(self, main):
		"Sests important values"
		self.sessions = {}
		self.nodes = {}
		self.sessionids = 0
		self.main = main

	def registerNode(self, name, desc, firststage, jid = None, public=False, args = {}):
		"""Registers a commands node to client
@type name: unicode
@param name: name of the node
@type desc: unicode
@param desc: human-readable name of the node
@type jid: unicode
@param jid: full jid of node, if not set it is the same as client jid
@type public: bool or list
@param public: if other jids (not only resources) can execute the node"""
		if jid == None:
			jid = unicode(self.main.client.jid.full())
		self.main.client.registerFeature("http://jabber.org/protocol/commands", name, identity={"category":"automation","type":"command-node","name":desc})
		self.main.client.registerFeature("jabber:x:data", name)
		self.main.client.discoitems["http://jabber.org/protocol/commands"].append({"jid":jid,"name":desc,"node":name})
		self.main.client.discoitems[name] = []
		self.nodes[name] = [desc, firststage, jid, public, args]
	
	def unregisterNode(self, name, desc, jid = None):
		if jid == None:
			jid = unicode(self.main.client.jid.full())
		try:
			del self.nodes[name]
			self.main.client.discoitems["http://jabber.org/protocol/commands"].remove({"jid":jid,"name":desc,"node":name})
			del self.main.client.discoitems[name]
			self.main.client.unregisterFeature("http://jabber.org/protocol/commands", name)
		except:
			log.err('unregistering command unsuccesful')
		

	def startSession(self, node, jid, fid):
		"Starts new session (node)"
		self.sessionids += 1
		self.sessions[unicode(self.sessionids)] = Session(self.main, node, self.nodes[node][1], jid, unicode(self.sessionids), fid, self.nodes[node][4])

	#def commandsList(self, el):
	#	self.main.client.disp(el["id"])
	#	iq = IQ(self.main.client.xmlstream, "result")
	#	iq["id"] = el["id"]
	#	iq["to"] = el["from"]
	#	query = iq.addElement("query", "http://jabber.org/protocol/disco#items")
	#	query["node"] = "http://jabber.org/protocol/commands"
	#	for node in self.nodes.keys():
	#		item = query.addElement("item")
	#		item["node"] = node
	#		item["name"] = self.nodes[node][0]
	#		item["jid"] = self.nodes[node][2]
	#	iq.send()
	#	self.main.client.disp(iq["id"])
